#include <hc595.h>

hc595::hc595(unsigned char latch,unsigned char clock,unsigned char data,unsigned char  num)
{
  latchPin=latch;
  clockPin=clock;
  dataPin=data;
  Buffer=(unsigned char*)malloc(num);  // num is number of 74HC595 connected
  for(int i=0;i<num;i++) Buffer[i]=0;
  pinMode(latchPin, OUTPUT);
  pinMode(dataPin, OUTPUT); 
  pinMode(clockPin, OUTPUT);
  num_595=num;
}

hc595::~hc595()
{
 free(Buffer);
 Buffer=0;
}

void hc595::PulseE(unsigned char num)
{
  bitWrite(Buffer[num],3,LOW); // set enable pin to low
  Send595();
  delayMicroseconds(1);   // >450ns
  bitWrite(Buffer[num],3,HIGH); // set enable pin to  high
  Send595();
  delayMicroseconds(1);    //  >450ns
  bitWrite(Buffer[num],3,LOW); // set enable pin to low
  Send595();
  delayMicroseconds(40);   //  > 37us
}

void hc595::Send595(void)
{
 int i;
 digitalWrite(latchPin, LOW);
 for(i=0;i<num_595;i++) shiftOut(dataPin, clockPin, MSBFIRST, Buffer[num_595-1-i]);
 digitalWrite(latchPin, HIGH);


} 


 

void hc595::Send595Pin(unsigned char val,unsigned char display)
{
  Set595Pin(val,display);
  Send595();
}


void hc595::Set595Pin(bool D7,bool D6,bool D5,bool D4,bool D3,bool D2,bool D1,bool D0,unsigned char num)
{
 bitWrite(Buffer[num],7,D7);
 bitWrite(Buffer[num],6,D6);
 bitWrite(Buffer[num],5,D5);
 bitWrite(Buffer[num],4,D4);
 bitWrite(Buffer[num],3,D3);
 bitWrite(Buffer[num],2,D2);
 bitWrite(Buffer[num],1,D1); 
 bitWrite(Buffer[num],0,D0);   
}

void hc595::Set595Pin(unsigned char val,unsigned char num)
{
 Buffer[num]=val;
}

 void hc595::Lcd_SetFreePin(unsigned char pin, bool status,unsigned char sr)
 {
  bitWrite(Buffer[sr],pin,status);
  Send595();
 }


void hc595::DisplayReset(unsigned char options,unsigned char num)
{
  if(!options) options=LCD595_BASIC_DISPLAY_INIT | LCD595_USEFONT_5X10 | LCD595_MORELINES; 
  //procedura di reset
  delayMicroseconds(50000); // attesa iniziale....40ms ( ho usato tempi abbondanti
 // Buffer[num]=0;
   
 SetLcdPin(0,0,1,1,num); // invio 0011
 PulseE(num);
 delayMicroseconds(4500); // 4.1ms
  
  SetLcdPin(0,0,1,1,num); // reinvio 0011
  PulseE(num); 
  delayMicroseconds(150); // 100us
  
  SetLcdPin(0,0,1,1,num); // reinvio 0011
  PulseE(num); 
  
  SetLcdPin(0,0,1,0,num); // ...settiamo i 4 bit...
  PulseE(num);  


  SendLcdCommand(options,num);  // Inizializzazione con font e righe : 42    0 0 1 DL N F 0 0 (DL: 0 (4bit), N=1 (due righe)  F:5x8 (0) vs 15x10 (1))
  SendLcdCommand(LCD595_DISPLAY_ON,num);  // accendo il display
  SendLcdCommand(LCD595_DISPLAY_CLEAR,num);  // clear
  
}
 /*
void hc595::DisplayClean(unsigned char sr)
{
 SendLcdCommand(LCD595_DISPLAY_CLEAR,sr);
}   */

void hc595::DisplayWrite(const char *buff,unsigned char display)
{
  int ln=strlen(buff);
  int i;
  for(i=0;i<ln;i++) {DisplayChar(buff[i],display);}

}

void hc595::DisplayChar(unsigned char buff,unsigned char display)
{
  //scrivo una lettera OPPURE invio dati generici per altri scopi
  SetLcdRS(HIGH,display); // porto RS ad HIGH per indicare l'invio di DATI
  SetHLcd(buff,display);
  PulseE(display);
  SetLLcd(buff,display);
  PulseE(display);
  delayMicroseconds(50); // 37us
}

void hc595::SetLcdPin(bool D7,bool D6,bool D5,bool D4,unsigned char num)
{
 bitWrite(Buffer[num],7,D7);
 bitWrite(Buffer[num],6,D6);
 bitWrite(Buffer[num],5,D5);
 bitWrite(Buffer[num],4,D4);
}



void hc595::SetCursor(unsigned char X, unsigned char Y,unsigned char type,unsigned char display)
{
  int row_offsets[] = { 0x00, 0x40, 0x14, 0x54 };     //type 1
  if(type==2) {row_offsets[2]=0x10; row_offsets[3]=0x50;}

  if(Y>3) Y=Y%4;
  unsigned char val;
  val=(0x80 | (X + row_offsets[Y]));
  SendLcdCommand(val,display);
}


void hc595::SetDDRAM_Address(unsigned char address,unsigned char display)
{
  unsigned char val=(0x80 | address);
  SendLcdCommand(val,display);
}

void hc595::CreateChar(unsigned char address,unsigned char map[],unsigned char display)
{
  address &= 0x7; // we only have 8 locations 0-7
  SendLcdCommand(0x40 | (address << 3),display);
  for (int i=0; i<8; i++) DisplayChar(map[i],display);

}
 
 void hc595::SendLcdCommand(unsigned char val,unsigned char display)
 {
   SetLcdRS(LOW,display);
   SetHLcd(val,display);
   PulseE(display);
   SetLLcd(val,display);
   PulseE(display);
   delayMicroseconds(50); // 37us 
   if(val==LCD595_DISPLAY_CLEAR) delay(2); //1.52ms    
}
 
 void hc595::SetLcdRS(bool Val,unsigned char display)
 {
  bitWrite(Buffer[display],1,Val); // porto RS ad HIGH per indicare l'invio di DATI o LOW per i comandi

 }
 
 void hc595::SetHLcd(unsigned char val,unsigned char display)
 {
  SetLcdPin(bitRead(val,7),bitRead(val,6),bitRead(val,5),bitRead(val,4),display); // primi 4 
 }

 void hc595::SetLLcd(unsigned char val,unsigned char display)
 {
  SetLcdPin(bitRead(val,3),bitRead(val,2),bitRead(val,1),bitRead(val,0),display); // ultimi 4 
 } 

 

